use crate::chunk::{Chunk, ConversionError, OptCode};
use crate::value::Value;
use std::env;

pub fn trace_enabled() -> bool {
    match env::var("DEBUG_TRACE_EXECUTION") {
        Ok(value) => match value.to_lowercase().as_str() {
            "true" | "1" | "yes" | "on" => true,
            "false" | "0" | "no" | "off" => false,
            _ => false,
        },
        Err(_e) => false,
    }
}
pub fn disassemble_chunk(chunk: &Chunk, name: &str) {
    // println!("{:?}", chunk);
    println!("== {} ==", name);
    let mut offset = 0;
    while offset < chunk.code.len() {
        offset = disassemble_instruction(&chunk, offset);
    }
}
pub fn disassemble_instruction(chunk: &Chunk, offset: usize) -> usize {
    print!("{:04} ", offset);
    if offset > 0 && chunk.lines[offset] == chunk.lines[offset - 1] {
        print!("   | ");
    } else {
        print!("{:>4} ", chunk.lines[offset]);
    }
    let instruction: Result<OptCode, ConversionError> = chunk.code[offset].try_into();
    match instruction {
        Ok(code) => match code {
            OptCode::OpConstant => {
                return constant_instruction("OpConstant", chunk, offset);
            }
            OptCode::OpReturn => {
                return simple_instruction("OpReturn", offset);
            }
            OptCode::OpNegate => {
                return simple_instruction(&format!("{:}", code), offset);
            }
            OptCode::OpAdd | OptCode::OpSubstract | OptCode::OpMultiply | OptCode::OpDivide => {
                return simple_instruction(&format!("{:}", code), offset);
            }
        },
        Err(e) => {
            println!("{:}: '{:}'", e.message, e.invalid_value);
            return offset + 1;
        }
    }
}

fn simple_instruction(name: &str, offset: usize) -> usize {
    println!("{}", name);
    return offset + 1;
}
fn constant_instruction(name: &str, chunk: &Chunk, offset: usize) -> usize {
    let constant: u8 = chunk.code[offset + 1];
    print!("{:<16} {:>4} ", name, constant);
    print_value(chunk.constants[constant as usize]);
    println!();
    return offset + 2;
}
pub fn print_value(value: Value) {
    print!("'{:}'", value);
}
