use clap::{Arg, ArgAction, Command};
use mox::chunk::Chunk;
use mox::scanner::Scanner;
use mox::vm::InterpretResult;
use mox::vm::VM;
use std::fs;
// use std::io;
// use std::io::Write;
use std::process;
use std::process::exit;

fn main() {
    let app = Command::new("mox")
        .version("1.0")
        .author("publicmatt")
        .about("mox interpreter!!")
        .arg(
            Arg::new("file")
                .short('f')
                .long("file")
                .action(ArgAction::Set)
                .value_name("FILE")
                .help("file to run"),
        )
        .arg(
            Arg::new("command")
                .short('c')
                .long("command")
                .action(ArgAction::Set)
                .value_name("COMMAND")
                .help("command to run"),
        )
        .arg(
            Arg::new("scan")
                .short('s')
                .long("scan")
                .action(ArgAction::SetTrue)
                .value_name("SCAN")
                .help("scan only"),
        );

    let matches = app.get_matches();

    if let Some(source) = matches.get_one::<String>("command") {
        if let Some(_) = matches.get_one::<bool>("scan") {
            scan_content(source);
        } else {
            run_content(source);
        }
    } else if let Some(file) = matches.get_one::<String>("file") {
        let source = fs::read_to_string(file).unwrap_or_else(|err| {
            eprintln!("Error reading '{}': {}", file, err);
            process::exit(74);
        });
        println!("{}", source);
        if let Some(_) = matches.get_one::<bool>("scan") {
            scan_content(&source);
        } else {
            run_content(&source);
        }
    } else {
        todo!("repl not done yet")
        // repl();
    }
}

// fn repl() {
//     let mut input = String::new();
//     let mut vm: VM = VM::new();
//     let mut chunk: Chunk = Chunk::new();
//     loop {
//         input.clear();
//         print!("> ");
//         io::stdout().flush().unwrap();
//
//         match io::stdin().read_line(&mut input) {
//             Ok(bytes) => {
//                 if bytes == 0 || input.trim() == "exit" {
//                     println!("Bye!");
//                     break;
//                 }
//                 vm.interpret(&input, &mut chunk);
//             }
//             Err(_error) => {
//                 continue;
//             }
//         }
//     }
// }

fn scan_content(source: &str) {
    let input = source.to_owned();
    let mut scanner = Scanner::new(&input);
    scanner.compile();
}
fn run_content(source: &str) {
    let mut vm: VM = VM::new();
    let mut chunk: Chunk = Chunk::new();
    match vm.interpret(source, &mut chunk) {
        InterpretResult::InterpretOk => exit(0),
        InterpretResult::InterpretCompileError => exit(65),
        InterpretResult::InterpretRuntimeError => exit(70),
    }
}
